clear
clc
close all



%% READ DATA
disp('reading files... (it takes some time)');
DATA=load('result.out'); 
PARAMETERS=load('parameters.dat');
GEOMETRY=load('geometry.dat');

Nd=size(DATA,1);   %number of data
Np=DATA(Nd,2)+1;   %number of pedestrians
Nt=DATA(Nd,1)+1;   %number of time steps
Dt=PARAMETERS(2);  %time step
contactdist=PARAMETERS(6);
physicaldist=PARAMETERS(8);

% rows:agents, columns:time
POSx=       reshape(DATA(:,3),[Np,Nt]);     %position (x)
POSy=       reshape(DATA(:,4),[Np,Nt]);     %position (y)
ISCONTACTED=reshape(DATA(:,5),[Np,Nt]);
ISPUSHED=   reshape(DATA(:,6),[Np,Nt]);
ISWAITING=  reshape(DATA(:,7),[Np,Nt]);
ISDEAD=     reshape(DATA(:,8),[Np,Nt]);

L=GEOMETRY(1);  %corridor length
H=GEOMETRY(2);  %corridor height

S=5;
elle=1;

disp('done');



%% PLOT SIMULATION
hh=figure(1);
%set(gcf, 'Position', 1*[10, 200, 1300, floor(1300/3)])
axis([-1 L+1 -1 H+1])
pbaspect([(L+1)/(H+1) 1 1]);
ax = gca;
ax.FontSize = 6; 
hold on;
set(gca, 'Visible', 'on')
hh.Renderer='Painters';

for n=1:10:5500
  
  cla;
  
  %geometry
  plot([0 L],[0 0],'k','LineWidth',2')
  plot([0 L],[H H],'k','LineWidth',2')
  plot([S S],[-0.5 H+0.5],'g','LineWidth',3') 
  plot([S-elle S-elle],[-0.5 H+0.5],'g:','LineWidth',2') 
  plot([S+elle S+elle],[-0.5 H+0.5],'g:','LineWidth',2') 


  %normal
  app=( (ISWAITING(:,n)==0) & (ISDEAD(:,n)==0) );
  X=POSx(app,n);
  Y=POSy(app,n);
  centers=[X Y];
  radii_in=0.5*physicaldist*ones(length(X),1);
  radii_out=0.5*contactdist*ones(length(X),1);
  viscircles(centers,radii_in,'Color','b');
  %viscircles(centers,radii_out,'Color','b');
      
  %contacted
  app=( (ISCONTACTED(:,n)==1) & (ISPUSHED(:,n)==0) );
  X=POSx(app,n);
  Y=POSy(app,n);
  centers=[X Y];
  radii_in=0.5*physicaldist*ones(length(X),1);
  %viscircles(centers,radii_in,'Color',[255 165 0]/255);
  
  %pushed
  app=(ISPUSHED(:,n)==1);
  X=POSx(app,n);
  Y=POSy(app,n);
  centers=[X Y];
  radii_in=0.5*physicaldist*ones(length(X),1);
  viscircles(centers,radii_in,'Color','r');
  
  %dead
  app=(ISDEAD(:,n)==1);
  X=POSx(app,n);
  Y=POSy(app,n);
  centers=[X Y];
  radii_in=0.5*physicaldist*ones(length(X),1);
  %viscircles(centers,radii_in,'Color','k');
  
  %waiting
  app=(ISWAITING(:,n)==1);
  X=POSx(app,n);
  Y=POSy(app,n);
  centers=[X Y];
  radii_in=0.2*physicaldist*ones(length(X),1);
  %viscircles(centers,radii_in,'Color','k');
  
  xlabel(['t=' num2str(n*Dt) 's'] ,'FontSize',14);
  pause(0.01);

end



%% FLUX-DENSIY ANALYSIS
for n=1:Nt-1
  flux(n)=0;
  density(n)=0;
  for k=1:Np
    if ISWAITING(k,n)==0
      if (POSx(k,n)<S) && (POSx(k,n+1)>S) 
        flux(n)=flux(n)+1;
      end
      if (POSx(k,n)<S+elle) && (POSx(k,n)>S-elle)
        density(n)=density(n)+1;
      end
    end
  end
end
flux=(flux/Dt)/H; %flux/time/space 
density=density/(2*elle*H); %density in m2 

figure
subplot(1,2,1); plot(flux);    xlabel('time step n'); ylabel('flux')
subplot(1,2,2); plot(density); xlabel('time step n'); ylabel('density')

%average for better visualization
fluxMM=movmean(flux,300); 
densityMM=movmean(density,50);

figure
subplot(1,2,1); plot(fluxMM);    xlabel('time step n'); ylabel('averaged flux')
subplot(1,2,2); plot(densityMM); xlabel('time step n'); ylabel('averaged density')

figure;
plot(densityMM,fluxMM,'LineWidth',2)
axis([0 11 0 2.5]);
xticks(1:11)
title('Fundamental diagram'); xlabel('density (ped/m^2)'); ylabel('flux (ped/m/s)');

figure;
k=2;
for n=1:Nt-1
  velx(n)=(POSx(k,n+1)-POSx(k,n))/Dt;
end
velx(velx<-1)=0;
velxMM=movmean(velx,300);
plot(1:5500,velxMM(1:5500),'LineWidth',2)
xlabel('time step n'); ylabel('velocity (m/s)');
axis([1 5500 0 0.9])
